@@ -41,7 +41,6 @@ namespace nux
   */
   static bool TimeIsGreater(TimeStruct t1, TimeStruct t2);
   static void TimeRightNow(TimeStruct *tv);
-  static void Addmillisecs(TimeStruct *tv, unsigned int milliseconds);
 //static unsigned int TimeDiff(TimeStruct t1, TimeStruct t2);
 
   static NThreadSafeCounter TimerUID = 0x01234567;
@@ -55,6 +54,7 @@ namespace nux
 
     //! Delay before the callback expires
     TimeStruct      when;
+    gint64          ms_time; // milliseconds 
     void            *CallbackData;
     TimeOutSignal    *timeout_signal;
 
@@ -67,16 +67,18 @@ namespace nux
     int             Period;                 //!< The periode of the timer interuption(in milliseconds).
     int             Duration;               //!< How long the timer will be running from start to finish(in milliseconds);
     int             ElapsedTime;            //!< Elapsed time during execution(in milliseconds).
-    bool            MarkedForRemoval;
+    bool            marked_for_removal_;
     BaseWindow      *Window;                 //!< BaseWindow from where the timer was created.
     TimerObject     *next;
     TimerObject     *prev;
     unsigned int           glibid;
     unsigned int           uid;
+    TimerHandler::TimerState      state_;
   };
 
   TimerObject::TimerObject()
   {
+    ms_time         = 0;
     Type            = 0;
     CallbackData    = 0;
     timeout_signal   = 0;
@@ -86,12 +88,13 @@ namespace nux
     ElapsedTime     = 0;
     ScheduledIteration       = 0;
     ProgressIterationCount  = 0;
-    MarkedForRemoval = 0;
+    marked_for_removal_ = 0;
     Window          = 0;
     next            = 0;
     prev            = 0;
     glibid          = 0;
     uid             = 0;
+    state_          = TimerHandler::TIMER_STATE_STOPED;
   }
 
   TimerHandle::TimerHandle()
@@ -168,14 +171,13 @@ namespace nux
 ////////////////////////////////////////////////////
   TimerHandler::TimerHandler(WindowThread* window_thread)
   : window_thread_(window_thread)
+  , is_processing_timers_(false)
+  , timer_object_queue_(NULL)
   {
-    m_timer_object_queue = 0;
-    m_IsProceesingTimers = false;
   }
 
   TimerHandler::~TimerHandler()
   {
-
   }
 
   void TimerHandler::StartEarlyTimerObjects()
@@ -184,8 +186,7 @@ namespace nux
     for (it = _early_timer_objects.begin(); it != _early_timer_objects.end(); it++)
     {
       TimerObject *timer_object = *it;
-      TimeRightNow(&timer_object->when);
-      Addmillisecs(&timer_object->when, timer_object->Period);
+      timer_object->ms_time = g_get_monotonic_time() / 1000;
 
 #if (defined(NUX_OS_LINUX) || defined(NUX_USE_GLIB_LOOP_ON_WINDOWS)) && (!defined(NUX_DISABLE_GLIB_LOOP))
         timer_object->glibid = GetWindowThread()->AddTimeout(timer_object->Period);
@@ -195,17 +196,17 @@ namespace nux
     _early_timer_objects.clear();
   }
 
-  TimerHandle TimerHandler::AddTimerHandler(unsigned int Period, TimeOutSignal *timeout_signal, void *Data, WindowThread* window_thread)
+  TimerHandle TimerHandler::AddOneShotTimer(unsigned int Period, TimeOutSignal* timeout_signal, void* Data, WindowThread* window_thread)
   {
     TimerObject *timer_object = new TimerObject();
 
-    TimeRightNow(&timer_object->when);
-    Addmillisecs(&timer_object->when, Period);
+    timer_object->ms_time = g_get_monotonic_time() / 1000;
 
     timer_object->CallbackData  = Data;
     timer_object->timeout_signal = timeout_signal;
     timer_object->Period        = Period;
     timer_object->Type          = TIMERTYPE_PERIODIC;
+    timer_object->state_        = TimerHandler::TIMER_STATE_RUNNING;
     if (window_thread)
       timer_object->Window        = window_thread->GetWindowCompositor().GetProcessingTopView();
     else
@@ -225,10 +226,10 @@ namespace nux
         _early_timer_objects.push_back(timer_object);
         // Probably trying to set a timeout before Glib main context and loop have been created.
         // Sometimes later, this timer will be examined when ExecTimerHandler is called.
-        // This happens when trying to set a callback before the mainloop has been initialized.
+        // This happens when trying to set a callback before the main loop has been initialized.
       }
 
-      //nuxDebugMsg("[TimerHandler::AddTimerHandler] Adding Timeout ID: %d", timer_object->glibid);
+      //nuxDebugMsg("[TimerHandler::AddOneShotTimer] Adding Timeout ID: %d", timer_object->glibid);
     }
 #endif
 
@@ -236,17 +237,18 @@ namespace nux
     return handle;
   }
 
-  TimerHandle TimerHandler::AddPeriodicTimerHandler(unsigned int Period, int Duration, TimeOutSignal *timeout_signal, void *Data)
+  TimerHandle TimerHandler::AddDurationTimer(unsigned int Period, int Duration, TimeOutSignal *timeout_signal, void *Data)
   {
-    TimerObject *timer_object = new TimerObject();
-    TimeRightNow(&timer_object->when);
-    Addmillisecs(&timer_object->when, Period);
+    TimerObject* timer_object = new TimerObject();
+
+    timer_object->ms_time = g_get_monotonic_time() / 1000;
     timer_object->CallbackData = Data;
     timer_object->timeout_signal = timeout_signal;
 
-    timer_object->Period = Period;
-    timer_object->Duration = (Duration < 0) ? -1 : Duration;
-    timer_object->Type = TIMERTYPE_DURATION;
+    timer_object->Period    = Period;
+    timer_object->Duration  = (Duration < 0) ? -1 : Duration;
+    timer_object->Type      = TIMERTYPE_DURATION;
+    timer_object->state_    = TimerHandler::TIMER_STATE_RUNNING;
     AddHandle(timer_object);
 
 #if (defined(NUX_OS_LINUX) || defined(NUX_USE_GLIB_LOOP_ON_WINDOWS)) && (!defined(NUX_DISABLE_GLIB_LOOP))
@@ -261,7 +263,7 @@ namespace nux
         // This happens when trying to set a callback before the mainloop has been initialized.
       }
 
-      //nuxDebugMsg("[TimerHandler::AddTimerHandler] Adding Timeout ID: %d", timer_object->glibid);
+      //nuxDebugMsg("[TimerHandler::AddOneShotTimer] Adding Timeout ID: %d", timer_object->glibid);
     }
 #endif
 
@@ -269,17 +271,18 @@ namespace nux
     return handle;
   }
 
-  TimerHandle TimerHandler::AddCountIterationTimerHandler(unsigned int Period, int NumberOfIterations, TimeOutSignal *timeout_signal, void *Data)
+  TimerHandle TimerHandler::AddIterativeTimer(unsigned int Period, int NumberOfIterations, TimeOutSignal *timeout_signal, void *Data)
   {
-    TimerObject *timer_object = new TimerObject();
-    TimeRightNow(&timer_object->when);
-    Addmillisecs(&timer_object->when, Period);
+    TimerObject* timer_object = new TimerObject();
+
+    timer_object->ms_time = g_get_monotonic_time() / 1000;
     timer_object->CallbackData = Data;
     timer_object->timeout_signal = timeout_signal;
 
-    timer_object->Period = Period;
+    timer_object->Period              = Period;
     timer_object->ScheduledIteration  = (NumberOfIterations < 0) ? -1 : NumberOfIterations;
-    timer_object->Type = TIMERTYPE_ITERATION;
+    timer_object->Type                = TIMERTYPE_ITERATION;
+    timer_object->state_              = TimerHandler::TIMER_STATE_RUNNING;
     AddHandle(timer_object);
 
 #if (defined(NUX_OS_LINUX) || defined(NUX_USE_GLIB_LOOP_ON_WINDOWS)) && (!defined(NUX_DISABLE_GLIB_LOOP))
@@ -294,7 +297,7 @@ namespace nux
         // This happens when trying to set a callback before the mainloop has been initialized.
       }
 
-      //nuxDebugMsg("[TimerHandler::AddTimerHandler] Adding Timeout ID: %d", timer_object->glibid);
+      //nuxDebugMsg("[TimerHandler::AddOneShotTimer] Adding Timeout ID: %d", timer_object->glibid);
     }
 #endif
     TimerHandle handle(timer_object);
@@ -302,7 +305,7 @@ namespace nux
   }
 
 // Sort timers and add them to the queue
-  TimerObject *TimerHandler::AddHandle(TimerObject *timer_object)
+  TimerObject *TimerHandler::AddHandle(TimerObject* timer_object)
   {
     if (timer_object == NULL)
       return NULL;
@@ -313,26 +316,26 @@ namespace nux
 
     // If the queue is empty or the new timer will expire sooner than the first timer in the queue
     // then add the new timer at the start of the queue.
-    if ((m_timer_object_queue == NULL) || TimeIsGreater(m_timer_object_queue->when, timer_object->when))
+    if ((timer_object_queue_ == NULL) || (timer_object_queue_->ms_time > timer_object->ms_time))
     {
       // Add the timer timer_object at the head of the queue
-      timer_object->next = m_timer_object_queue;
+      timer_object->next = timer_object_queue_;
 
-      if (m_timer_object_queue)
-        m_timer_object_queue->prev = timer_object;
+      if (timer_object_queue_)
+        timer_object_queue_->prev = timer_object;
 
       timer_object->prev = 0;
-      m_timer_object_queue = timer_object;
+      timer_object_queue_ = timer_object;
 
       return timer_object;
     }
 
-    TimerObject *tmp = m_timer_object_queue;
+    TimerObject *tmp = timer_object_queue_;
 
     while (tmp->next != NULL)
     {
       // Is the time to wait for tmp->next to expire smaller than for timer_object
-      if (TimeIsGreater(timer_object->when, tmp->next->when))
+      if (timer_object->ms_time > tmp->next->ms_time)
       {
         // keep searching
         tmp = tmp->next;
@@ -356,7 +359,7 @@ namespace nux
   unsigned int TimerHandler::GetNumPendingHandler()
   {
     unsigned int count = 0;
-    TimerObject *head = m_timer_object_queue;
+    TimerObject *head = timer_object_queue_;
 
     while (head)
     {
@@ -367,20 +370,20 @@ namespace nux
     return count;
   }
 
-  bool TimerHandler::RemoveTimerHandler(TimerHandle &timer_object)
+  bool TimerHandler::RemoveTimerHandler(TimerHandle &handle)
   {
-    NUX_RETURN_VALUE_IF_NULL(timer_object.m_d, false);
-    NUX_RETURN_VALUE_IF_NULL(m_timer_object_queue, false);
+    NUX_RETURN_VALUE_IF_NULL(handle.m_d, false);
+    NUX_RETURN_VALUE_IF_NULL(timer_object_queue_, false);
 
-    TimerObject *tmp;
+    TimerObject* tmp;
 
-    tmp = m_timer_object_queue;
+    tmp = timer_object_queue_;
 
     while (tmp)
     {
-      if ((tmp == timer_object.m_d) && (tmp->uid == timer_object.m_d->uid))
+      if ((tmp == handle.m_d) && (tmp->uid == handle.m_d->uid))
       {
-        if (!m_IsProceesingTimers)
+        if (!is_processing_timers_)
         {
           if (tmp->next)
             tmp->next->prev = tmp->prev;
@@ -388,14 +391,14 @@ namespace nux
           if (tmp->prev)
             tmp->prev->next = tmp->next;
 
-          if ((timer_object.m_d == m_timer_object_queue) && (timer_object.m_d->uid == m_timer_object_queue->uid))
-            m_timer_object_queue = timer_object.m_d->next;
+          if ((handle.m_d == timer_object_queue_) && (handle.m_d->uid == timer_object_queue_->uid))
+            timer_object_queue_ = handle.m_d->next;
 
-          NUX_SAFE_DELETE(timer_object.m_d);
+          NUX_SAFE_DELETE(handle.m_d);
         }
         else
         {
-          timer_object.m_d->MarkedForRemoval = true;
+          handle.m_d->marked_for_removal_ = true;
         }
 
         return true;
@@ -407,44 +410,119 @@ namespace nux
     return false;
   }
 
-#if (defined(NUX_OS_LINUX) || defined(NUX_USE_GLIB_LOOP_ON_WINDOWS)) && (!defined(NUX_DISABLE_GLIB_LOOP))
-  int TimerHandler::ExecTimerHandler(unsigned int timer_id)
-#else
-  int TimerHandler::ExecTimerHandler()
-#endif
+  bool TimerHandler::PauseTimer(TimerHandle& handle)
   {
-    NUX_RETURN_VALUE_IF_NULL(m_timer_object_queue, 0);
+    NUX_RETURN_VALUE_IF_NULL(handle.m_d, false);
+    NUX_RETURN_VALUE_IF_NULL(timer_object_queue_, false);
+
+    if (handle.m_d->state_ != TimerHandler::TIMER_STATE_RUNNING)
+    {
+      return false;
+    }
+
+    TimerObject* tmp;
+
+    // Pointer to the start of the queue.
+    tmp = timer_object_queue_;
+
+    // Search for the timer handle
+    while (tmp)
+    {
+      if ((tmp == handle.m_d) && (tmp->uid == handle.m_d->uid))
+      {
+        handle.m_d->state_ = TimerHandler::TIMER_STATE_PAUSED;
+
+
+        if (!is_processing_timers_)
+        {
+          gint64 ms_time_now = g_get_monotonic_time() / 1000;
+          
+          if (handle.m_d->Type == TIMERTYPE_PERIODIC)
+          {
+            handle.m_d->ElapsedTime += (ms_time_now - handle.m_d->ms_time);
+            handle.m_d->ProgressDelta = float(handle.m_d->ElapsedTime) / float(handle.m_d->Period);
+
+            if (handle.m_d->Param + handle.m_d->ProgressDelta > 1.0f)
+              handle.m_d->ProgressDelta = 1.0f - handle.m_d->Param;
+
+            handle.m_d->Param = float(handle.m_d->ElapsedTime) / float(handle.m_d->Period);
+          }
+        }
+
+        return true;
+      }
+
+      tmp = tmp->next;
+    }
+    return false;
+  }
+
+  bool TimerHandler::ResumeTimer(TimerHandle& handle)
+  {
+    NUX_RETURN_VALUE_IF_NULL(handle.m_d, false);
+    NUX_RETURN_VALUE_IF_NULL(timer_object_queue_, false);
+
+    if (handle.m_d->state_ != TimerHandler::TIMER_STATE_PAUSED)
+    {
+      return false;
+    }
+
+    TimerObject* tmp;
+
+    // Pointer to the start of the queue.
+    tmp = timer_object_queue_;
+
+    // Search for the timer handle
+    while (tmp)
+    {
+      if ((tmp == handle.m_d) && (tmp->uid == handle.m_d->uid))
+      {
+        handle.m_d->state_ = TimerHandler::TIMER_STATE_RUNNING;
+
+        if (handle.m_d->Type == TIMERTYPE_PERIODIC)
+        {
+          handle.m_d->glibid = GetWindowThread()->AddTimeout(handle.m_d->Period * (1.0f - handle.m_d->Param));
+        }
+        else
+        {
+          handle.m_d->glibid = GetWindowThread()->AddTimeout(handle.m_d->Period);
+        }
+
+        handle.m_d->ms_time = g_get_monotonic_time() / 1000;
+
+        if (handle.m_d->glibid == 0)
+        {
+          _early_timer_objects.push_back(handle.m_d);
+        }
+        return true;
+      }
+
+      tmp = tmp->next;
+    }
+    return false;
+  }
+
+  int TimerHandler::ExecTimerHandler(unsigned int timer_id)
+  {
+    NUX_RETURN_VALUE_IF_NULL(timer_object_queue_, 0);
 
     bool repeat = false;
-    TimerObject *timer_object = m_timer_object_queue;
-    TimeStruct now;
+    // Pointer to the start of the timer handler queue.
+    TimerObject* timer_object = timer_object_queue_;
 
-    int timer_executed = 0;
-
-    TimeRightNow(&now);
-
-    m_IsProceesingTimers = true;
+    is_processing_timers_ = true;
 
     while (timer_object != NULL)
     {
-#if (defined(NUX_OS_LINUX) || defined(NUX_USE_GLIB_LOOP_ON_WINDOWS)) && (!defined(NUX_DISABLE_GLIB_LOOP))
-
-      if ((/*TimeIsGreater(now, timer_object->when) ||*/ (timer_object->glibid == timer_id)) && (timer_object->MarkedForRemoval == false))
-#else
-      if (TimeIsGreater(now, timer_object->when))
-#endif
+      if ((timer_object->glibid == timer_id) && (timer_object->marked_for_removal_ == false))
       {
-        long elaps = now.sec - timer_object->when.sec;
-        long uelaps = now.usec - timer_object->when.usec;
-
-        if (uelaps < 0)
+        if (timer_object->state_ != TIMER_STATE_RUNNING)
         {
-          uelaps += 1000000;
-          //nuxAssert(elaps > 0);
-          elaps -= 1;
+          return false;
         }
+        // Find the first timer object that 
 
-        timer_object->ElapsedTime += timer_object->Period; //elaps * 1000 + uelaps / 1000; // milliseconds
+        timer_object->ElapsedTime += timer_object->Period;
 
         if (timer_object->Type == TIMERTYPE_PERIODIC)
         {
@@ -484,15 +562,20 @@ namespace nux
           nuxAssertMsg(0, "[TimerHandler::ExecTimerHandler] Unknown timer type.");
         }
 
-        if (timer_object->Param > 1.0f)
-          timer_object->Param = 1.0f;
 
-        timer_object->MarkedForRemoval = false;
+        if (timer_object->Param > 1.0f)
+        {
+          // correction.
+          timer_object->Param = 1.0f;
+        }
+
+        timer_object->marked_for_removal_ = false;
 
         if (timer_object->timeout_signal != 0)
         {
+          // Execute the signal
           GetWindowThread()->GetWindowCompositor().SetProcessingTopView(timer_object->Window);
-          timer_object->timeout_signal->time_expires.emit(timer_object->CallbackData);
+          timer_object->timeout_signal->tick.emit(timer_object->CallbackData);
           GetWindowThread()->GetWindowCompositor().SetProcessingTopView(NULL);          
           // Reset glibid to 0. glibid is not null, if this element ever happened to be at the head of the queue
           // and we set a timer for it.
@@ -500,56 +583,68 @@ namespace nux
           //timer_object->glibid = 0;
         }
 
-        TimerObject *expired_handler = 0;
+        TimerObject* expired_handler = 0;
 
-        if (timer_object->MarkedForRemoval)
+        if (timer_object->marked_for_removal_)
         {
           // RemoveTimerHandler was called during the callback execution
           expired_handler = timer_object;
         }
         else if (timer_object->Type == TIMERTYPE_PERIODIC)
         {
+          // A one shot timer expires after the first execution.
           expired_handler = timer_object;
         }
         else if ((timer_object->Type == TIMERTYPE_DURATION) && (timer_object->Param >= 1.0f))
         {
+          // A timer delay timer expires after the duration of the timer as expired.
           expired_handler = timer_object;
         }
         else if ((timer_object->Type == TIMERTYPE_ITERATION) && (timer_object->ProgressIterationCount >= timer_object->ScheduledIteration))
         {
+          // An iterative timer expires after the timer as been executedN times.
           expired_handler = timer_object;
         }
 
-        if (expired_handler)
+        if (!expired_handler && (timer_object->state_ == TIMER_STATE_PAUSED))
         {
+          // The state has been changed to "paused".
+          return false;
+        }
+
+        if(expired_handler)
+        {
+          GetWindowThread()->GetWindowCompositor().SetProcessingTopView(timer_object->Window);
+          timer_object->timeout_signal->expired.emit(timer_object->CallbackData);
+          GetWindowThread()->GetWindowCompositor().SetProcessingTopView(NULL);          
+
+          timer_object->state_ = TIMER_STATE_STOPED;
+
           if (timer_object->next)
             timer_object->next->prev = timer_object->prev;
 
           if (timer_object->prev)
             timer_object->prev->next = timer_object->next;
 
-          if ((timer_object == m_timer_object_queue) && (timer_object->uid == m_timer_object_queue->uid))
+          if ((timer_object == timer_object_queue_) && (timer_object->uid == timer_object_queue_->uid))
           {
             // timer_object is the first element of the queue.
-            m_timer_object_queue = timer_object->next;
+            timer_object_queue_ = timer_object->next;
 
-            if (m_timer_object_queue)
+            if (timer_object_queue_)
             {
-              m_timer_object_queue->prev = 0;
+              timer_object_queue_->prev = 0;
             }
           }
 
           timer_object = timer_object->next;
-
           delete expired_handler;
         }
         else
         {
           repeat = true;
-          //timer_object = timer_object->next;
         }
 
-        timer_executed++;
         break;
       }
       else
@@ -558,24 +653,14 @@ namespace nux
       }
     }
 
-//     // Look at the head of the queue and set a glib timeout for the first element, if one wasn't set already.
-//     if (m_timer_object_queue && (m_timer_object_queue->glibid == 0))
-//     {
-//         // How long(in milliseconds) between now and the moment the timeout expires?
-//         unsigned int time_difference = TimeDiff(now, m_timer_object_queue->when);
-//
-//         m_timer_object_queue->glibid = GetWindowThread()->AddTimeout(time_difference);
-//         //nuxDebugMsg("[TimerHandler::ExecTimerHandler] Adding Timeout ID: %d", m_timer_object_queue->glibid);
-//     }
-
     // Purge handles that have been marked for removal
-    timer_object = m_timer_object_queue;
+    timer_object = timer_object_queue_;
 
     while (timer_object)
     {
-      if (timer_object->MarkedForRemoval)
+      if (timer_object->marked_for_removal_)
       {
-        TimerObject *expired_handler = timer_object;
+        TimerObject* expired_handler = timer_object;
 
         if (timer_object->next)
           timer_object->next->prev = timer_object->prev;
@@ -585,11 +670,11 @@ namespace nux
         else
         {
           // timer_object is the first element of the queue.
-          m_timer_object_queue = timer_object->next;
+          timer_object_queue_ = timer_object->next;
 
-          if (m_timer_object_queue)
+          if (timer_object_queue_)
           {
-            m_timer_object_queue->prev = 0;
+            timer_object_queue_->prev = 0;
           }
         }
 
@@ -602,14 +687,13 @@ namespace nux
       }
     }
 
-    m_IsProceesingTimers = false;
-    //return timer_executed;
+    is_processing_timers_ = false;
     return repeat;
   }
 
   bool TimerHandler::FindTimerHandle(TimerHandle &timer_object)
   {
-    TimerObject *tmp = m_timer_object_queue;
+    TimerObject *tmp = timer_object_queue_;
 
     while (tmp)
     {
@@ -630,7 +714,7 @@ namespace nux
     TimeStruct now;
     TimeStruct delay;
 
-    if (m_timer_object_queue == NULL)
+    if (timer_object_queue_ == NULL)
     {
       // The return value of this function is only valid if there _are_ timers active.
       return 0;
@@ -639,14 +723,14 @@ namespace nux
     {
       TimeRightNow(&now);
 
-      if (TimeIsGreater(now, m_timer_object_queue->when))
+      if (TimeIsGreater(now, timer_object_queue_->when))
       {
         return 0;
       }
       else
       {
-        delay.sec = m_timer_object_queue->when.sec - now.sec;
-        delay.usec = m_timer_object_queue->when.usec - now.usec;
+        delay.sec = timer_object_queue_->when.sec - now.sec;
+        delay.usec = timer_object_queue_->when.usec - now.usec;
 
         // make sure that usec cannot be less than -1000000 before applying this code
         if (delay.usec < 0)
@@ -721,12 +805,4 @@ namespace nux
 #endif
   }
 
-  void Addmillisecs(TimeStruct *tv, unsigned int milliseconds)
-  {
-    tv->usec += milliseconds * 1000;
-    tv->sec += tv->usec / 1000000;
-    tv->usec = tv->usec % 1000000;
-  }
-
-
 }
