@@ -20,6 +20,7 @@
  */
 
 #include "Color.h"
+#include "ColorPrivate.h"
 
 #include <cmath>
 #include <cstdlib>
@@ -75,6 +76,16 @@ namespace color
   , premultiplied_(false)
   {}
 
+  Color::Color(std::string const& hex)
+  : red(0.0f)
+  , green(0.0f)
+  , blue(0.0f)
+  , alpha(1.0f)
+  , premultiplied_(false)
+  {
+    HexToRGBA(hex, red, green, blue, alpha);
+  }
+
   Color Color::GetPremultiplied()
   {
     if (premultiplied_)
@@ -128,7 +139,7 @@ namespace color
     return (((std::rand() % 255) << 24) |
             ((std::rand() % 255) << 16) |
             ((std::rand() % 255) << 8) |
-            (std::rand() % 255));
+            (255));
   }
 
   Color operator + (Color const& lhs, Color const& rhs)
