@@ -22,7 +22,7 @@
 
 #include "NuxCore/NuxCore.h"
 
-#include "NuxImage/ImageSurface.h"
+#include "ImageSurface.h"
 
 #include "GpuDevice.h"
 #include "GLDeviceObjects.h"
@@ -153,6 +153,8 @@ namespace nux
   GraphicsEngine::GraphicsEngine(GraphicsDisplay& GlWindow, bool create_rendering_data)
   : _graphics_display(GlWindow)
   {
+
+    IOpenGLShaderProgram::SetShaderTracking(true);
     _scissor.x = 0;
     _scissor.y = 0;
     _clip_offset_x = 0;
@@ -718,7 +720,7 @@ namespace nux
     _clip_offset_y = 0;
 
     std::list<Point>::iterator it;
-    for (it = _clip_offset_stack.begin(); it != _clip_offset_stack.end(); it++)
+    for (it = _clip_offset_stack.begin(); it != _clip_offset_stack.end(); ++it)
     {
       _clip_offset_x += (*it).x;
       _clip_offset_y += (*it).y;
@@ -739,7 +741,7 @@ namespace nux
     _clip_offset_y = 0;
 
     std::list<Point>::iterator it;
-    for (it = _clip_offset_stack.begin(); it != _clip_offset_stack.end(); it++)
+    for (it = _clip_offset_stack.begin(); it != _clip_offset_stack.end(); ++it)
     {
       _clip_offset_x += (*it).x;
       _clip_offset_y += (*it).y;
@@ -789,7 +791,7 @@ namespace nux
       Matrix4 temp;
       std::list<Matrix4>::iterator it;
 
-      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); it++)
+      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); ++it)
       {
         temp = _model_view_matrix;
         _model_view_matrix = temp * (*it);
@@ -848,7 +850,7 @@ namespace nux
       Matrix4 temp;
       std::list<Matrix4>::iterator it;
 
-      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); it++)
+      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); ++it)
       {
         temp = _model_view_matrix;
         _model_view_matrix = (*it) * temp;
@@ -876,7 +878,7 @@ namespace nux
       Matrix4 temp;
       std::list<Matrix4>::iterator it;
 
-      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); it++)
+      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); ++it)
       {
         temp = _model_view_matrix;
         _model_view_matrix = temp * (*it);
@@ -894,7 +896,7 @@ namespace nux
       Matrix4 temp;
       std::list<Matrix4>::iterator it;
 
-      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); it++)
+      for (it = m_2DModelViewMatricesStack.begin(); it != m_2DModelViewMatricesStack.end(); ++it)
       {
         temp = _model_view_matrix;
         _model_view_matrix = temp * (*it);
@@ -1036,6 +1038,11 @@ namespace nux
     _projection_matrix.Orthographic(0, viewport_width, viewport_height, 0, -1.0f, 1.0f);
   }
 
+  void GraphicsEngine::SetOrthographicProjectionMatrix(int left, int right, int bottom, int top)
+  {
+    _projection_matrix.Orthographic(left, right, bottom, top, -1.0f, 1.0f);
+  }
+
   void GraphicsEngine::ResetProjectionMatrix()
   {
     _projection_matrix = Matrix4::IDENTITY();
